﻿using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Controls;
using ThinkGeo.Core;


namespace WpfTestbed
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private CustomWmsLayer _wms;
        private LayerOverlay staticOverlay;

        public MainWindow()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Add the WMS layer to the map
        /// </summary>
        private async void MapView_Loaded(object sender, RoutedEventArgs e)
        {
            // It is important to set the map unit first to either feet, meters or decimal degrees.
            MapView.MapUnit = GeographyUnit.Meter;

            // Create an overlay that we will add the layer to.
            staticOverlay = new LayerOverlay();
            staticOverlay.DrawingExceptionMode = DrawingExceptionMode.ThrowException;

            staticOverlay.ThrowingException += (sender, e) =>
            {
                TxtException.Text = e.Exception?.Message;
                e.Handled = true;
            };

            MapView.Overlays.Add(staticOverlay);
            staticOverlay.TileCache = new FileRasterTileCache("./cache", "ExceptionSample");

            _wms = new CustomWmsLayer(new Uri("http://geo.vliz.be/geoserver/Dataportal/ows?service=WMS&"));
            _wms.DrawingExceptionMode = DrawingExceptionMode.DrawException;

            _wms.Parameters.Add("LAYERS", "eurobis_grid_15m-obisenv");
            _wms.Parameters.Add("STYLES", "generic");
            _wms.OutputFormat = "image/png";
            _wms.Crs = "EPSG:3857";  // Coordinate system, typically EPSG:3857 for WMS with Spherical 
            _wms.TimeoutInSeconds = 2;
            staticOverlay.Layers.Add("wmsImageLayer", _wms);

            // Set the current extent to a local area.
            MapView.CurrentExtent = new RectangleShape(14114144.61573416, 5195304.319841703, 16392171.878052138, 3442348.4809069675);
            _ = MapView.RefreshAsync();
        }

        private void DrawingExceptionMode_Checked(object sender, RoutedEventArgs e)
        {
            var button = (RadioButton)sender;
            if (button.Content == null) return;
            TxtException.Text = "";
            switch (button.Content.ToString())
            {
                case "Throw Exception":
                    staticOverlay.DrawingExceptionMode = DrawingExceptionMode.ThrowException;
                    _wms.DrawingExceptionMode = DrawingExceptionMode.ThrowException;
                    _wms.DrawCustomException = false;
                    break;
                case "Customize Drawing Exception":
                    staticOverlay.DrawingExceptionMode = DrawingExceptionMode.DrawException;
                    _wms.DrawingExceptionMode = DrawingExceptionMode.DrawException;
                    _wms.DrawCustomException = true;
                    break;
                case "Draw Exception":
                default:
                    staticOverlay.DrawingExceptionMode = DrawingExceptionMode.DrawException;
                    _wms.DrawingExceptionMode = DrawingExceptionMode.DrawException;
                    _wms.DrawCustomException = false;
                    break;
            }
            _ = MapView.RefreshAsync();
        }

        public void Dispose()
        {
            // Dispose of unmanaged resources.
            MapView.Dispose();
            // Suppress finalization.
            GC.SuppressFinalize(this);
        }
    }

    public class CustomWmsLayer : WmsAsyncLayer
    {
        public bool DrawCustomException { get; set; }

        public CustomWmsLayer(Uri uri)
            : base(uri)
        {
        }
        protected override async Task DrawAsyncCore(GeoCanvas canvas, Collection<SimpleCandidate> labelsInAllLayers)
        {
            ValidatorHelper.CheckObjectIsNotNull(canvas, "view");
            ValidatorHelper.CheckLayerIsOpened(IsOpen);
            ValidatorHelper.CheckGeoCanvasIsInDrawing(canvas.IsDrawing);

            var wmsRequestExtent = canvas.CurrentWorldExtent;
            // Apply projection if need
            if (ProjectionConverter != null)
                wmsRequestExtent = ProjectionConverter.ConvertToInternalProjection(canvas.CurrentWorldExtent);

            GeoImage tempImage = null;
            try
            {
                PointShape centerPointShape = canvas.CurrentWorldExtent.GetCenterPoint();
                int canvasWidth = (int)(canvas.Width * canvas.ScaleFactor);
                int canvasHeight = (int)(canvas.Height * canvas.ScaleFactor);

                var imageBytes = await DownloadImageAsync(canvasWidth, canvasHeight, 1,
                    wmsRequestExtent, canvas.MapUnit, canvas.CancellationToken);

                if (canvas.CancellationToken.IsCancellationRequested || imageBytes == null)
                    throw new TaskCanceledException();

                tempImage = new GeoImage(imageBytes);

                // Check if the image needs to be projected
                if (ProjectionConverter != null && tempImage != null)
                {
                    // Project the downloaded image to the external projection
                    RasterProjectionResult projectedResult = ProjectionConverter.ConvertToExternalProjection(tempImage, wmsRequestExtent);
                    if (canvas.CancellationToken.IsCancellationRequested)
                        throw new TaskCanceledException();

                    // Dispose the original image and use the projected one
                    tempImage.Dispose();
                    tempImage = projectedResult.Image;
                }

                if (tempImage != null)
                {
                    if (Math.Abs(canvas.ScaleFactor - 1.0) < double.Epsilon)
                        canvas.DrawWorldImageWithoutScaling(tempImage, centerPointShape.X, centerPointShape.Y, DrawingLevel.LevelOne);
                    else
                        canvas.DrawWorldImage(tempImage, centerPointShape.X, centerPointShape.Y, (tempImage.Width / canvas.ScaleFactor), (tempImage.Height / canvas.ScaleFactor), DrawingLevel.LevelOne);
                }
            }
            finally
            {
                tempImage?.Dispose();
            }
        }

        protected override void DrawExceptionCore(GeoCanvas canvas, Exception e)
        {
            if (!DrawCustomException)
            {
                base.DrawExceptionCore(canvas, e);
            }
            else
            {
                // customize the drawing exception. Here below we draw the error in red on orange canvas.
                canvas.DrawArea(canvas.CurrentWorldExtent, GeoBrushes.LightOrange, DrawingLevel.LevelOne);
                canvas.DrawText("Customized Exception Message", new GeoFont("Arial", 10), GeoBrushes.Red, new[] { new ScreenPointF(canvas.Width / 2, canvas.Height / 2) }, DrawingLevel.LabelLevel);
            }

            throw new Exception();
        }
    }
}